`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    15:41:40 11/14/2014 
// Design Name: 
// Module Name:    keyboar_ctrl 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module keyboard_ctrl(
    input clk25,
    input clr,
    input PS2C,
    input PS2D,
	 output reg [7:0] keyval1,
	 output reg [7:0] keyval2,
	 output reg [7:0] keyval3,
	 output reg key_pressed
    );
	 
	 reg PS2Cf, PS2Df;
	 reg [7:0] ps2c_filter, ps2d_filter;
	 reg [10:0] shift1, shift2, shift3;
	 reg [3:0] bit_count;
	 reg [3:0] state;
	 
	 parameter start = 4'b0000, wtclklo1 = 4'b0001, wtclkhi1 = 4'b0010,
		getkey1 = 4'b0011, wtclklo2 = 4'b0100, wtclkhi2 = 4'b0101,
		getkey2 = 4'b0110, breakey = 4'b0111, wtclklo3 = 4'b1000, 
		wtclkhi3 = 4'b1001, getkey3 = 4'b1010, flag_key = 4'b1111;
	 parameter bit_count_max = 4'b1011; // 11
	
	//assign key_pressed = (state == flag_key);
	 always @ (posedge clk25 or posedge clr) begin
		if(clr == 1) begin
			ps2c_filter <= 1;
			ps2d_filter <= 1;
			PS2Cf <= 1;
			PS2Df <= 1;
		end
		else begin
			ps2c_filter[7] <= PS2C;
			ps2c_filter[6:0] <= ps2c_filter[7:1];
			ps2d_filter[7] <= PS2D;
			ps2d_filter[6:0] <= ps2d_filter[7:1];
			if(ps2c_filter == 8'hFF)
				PS2Cf <= 1;
			else
				if(ps2c_filter == 8'h0)
					PS2Cf <= 0;
			if(ps2d_filter == 8'hFF)
				PS2Df <= 1;
			else
				if(ps2d_filter == 8'h0)
					PS2Df <= 0;
		end
	end
	
	always @ (posedge clk25 or posedge clr) begin
		if(clr == 1) begin
			state <= start;
			bit_count <= 0;
			shift1 <= 0;
			shift2 <= 0;
			shift3 <= 0;
			keyval1 <= 0;
			keyval2 <= 0;
			keyval3 <= 0;
			key_pressed <= 0;
		end
		else
			case(state)
				start:
				begin
					key_pressed <= 0;
					if(PS2Df == 1)
						state <= start;
					else
						state <= wtclklo1;
				end
				wtclklo1:
				begin
					key_pressed <= 0;
					if(bit_count < bit_count_max)
						if(PS2Cf == 1)
							state <= wtclklo1;
						else begin
							state <= wtclkhi1;
							shift1 <= {PS2Df, shift1[10:1]};
						end
					else
						state <= getkey1;
				end
				wtclkhi1:
					if(PS2Cf == 0)
						state <= wtclkhi1;
					else begin
						state <= wtclklo1;
						bit_count <= bit_count + 1;
					end
				getkey1:
					begin
						keyval1 <= shift1[8:1];
						state <= wtclklo2;
						bit_count <= 0;
					end
				wtclklo2:
					if(bit_count < bit_count_max)
						if(PS2Cf == 1)
							state <= wtclklo2;
						else begin
							state <= wtclkhi2;
							shift2 <= {PS2Df, shift2[10:1]};
						end
					else
						state <= getkey2;
				wtclkhi2:
					if(PS2Cf == 0)
						state <= wtclkhi2;
					else begin
						state <= wtclklo2;
						bit_count <= bit_count + 1;
					end
				getkey2:
					begin
						keyval2 <= shift2[8:1];
						state <= breakey;
						bit_count <= 0;
					end
				breakey:
					if(keyval2 == 8'hF0)
						state <= wtclklo3;
					else
						if(keyval1 == 8'hE0)
							state <= wtclklo1;
						else
							state <= wtclklo2;
				wtclklo3:
					if(bit_count < bit_count_max)
						if(PS2Cf == 1)
							state <= wtclklo3;
						else begin
							state <= wtclkhi3;
							shift3 <= {PS2Df, shift3[10:1]};
						end
					else
						state <= getkey3;
				wtclkhi3:
					if(PS2Cf == 0)
						state <= wtclkhi3;
					else begin
						state <= wtclklo3;
						bit_count <= bit_count + 1;
					end
				getkey3:
					begin
						keyval3 <= shift3[8:1];
						state <= flag_key;
						bit_count <= 0;
					end
				flag_key: 
					begin
						key_pressed <= 1;
						state <= wtclklo1;
					end
				default ;
			endcase
		end


endmodule
